/* Parallax Mesmic 2125 Driver Header
By TJ Johnson
3/24/09

Version 1
Returns the accleration in clock counts of the pulse.
Based on code from the BS2 compatablity libary in spin. Thanks to
  *         Primary Author: Martin Hebel          *
  *       Electronic Systems Technologies         *
  *   Southern Illinois University Carbondale     *
  *            www.siu.edu/~isat/est              *
  *                                               *
  *   Additional Code from:                       *
  *     Andy Lindsay, Parallax                    *
  *     Paul B. Voss, Smith College               *
  *           www.science.smith.edu/cmet/    
  
  
See end of file for MIT use.  

NOTES:
The mesmic 2125 unit outputs a pulse high of the accleration in that direction
at 100hz. For a raw accleration in units of g it will vary according to your 
clock speed. See the parallax data sheet for more info.

You must call memicStart before calling either of the return funcitons.

Both return functions are blocking and use counter A of the cog they are called
from.


*/
// Header information
#include <propeller.h>



// Global Constants



// Global Variables
int pinXglobal, pinYglobal;//Global Variables to store pinx and y in.


//Function Protypes
		   //mesmicStart( pin x, pin y) 
		   //Setup the driver
		   void mesmicStart(int pinX, int pinY);
		   
		   //Return the time x pulse in clk counts
		   int rawX(void);
		   
		   //Return the time of y pulse in clk counts
		   int rawY(void);



//Functions

// Set up the IO pins
void mesmicStart(int pinX, int pinY)
{
CLR(DIRA, pinX); // Set pinX to input
CLR(DIRA, pinY); // Set pinY to input

pinXglobal = pinX; // save the pin to the global vars.
pinYglobal = pinY;



}



int rawX(void)
{
/*{
  Reads duration of Pulse on pin defined for state, returns duration in 1/clkFreq increments - 12.5nS at 80MHz
  Note: Absence of pulse can cause cog lockupe
    
    
*/
  int pinXMask; // Set a variable for a pin x mask
  
  SET(pinXMask, pinXglobal);  // Set the pinx pin high in the mask
  CTRA = 0; // Clear Counter A
  
  CTRA = (0b11010 << 26 ) | (0b001 << 23) | (0 << 9) | (pinXglobal); // set up counter, A level count
  FRQA = 1;
  WAITPNE( pinXMask, pinXMask);                       // Wait for opposite state ready
  PHSA = 0;                                           // Clear count
  WAITPEQ(pinXMask, pinXMask);                        // wait for pulse
  WAITPNE(pinXMask,pinXMask);                         // Wait for pulse to end
  return PHSA; 										  // Return duration as counts

  
  


}


int rawY(void)
{
/*{
  Reads duration of Pulse on pin defined for state, returns duration in 1/clkFreq increments - 12.5nS at 80MHz
  Note: Absence of pulse can cause cog lockupe
    
    
*/
  int pinYMask; // Set a variable for a pin x mask
  
  SET(pinYMask, pinYglobal);  // Set the pinx pin high in the mask
  CTRA = 0; // Clear Counter A
  
  CTRA = (0b11010 << 26 ) | (0b001 << 23) | (0 << 9) | (pinYglobal); // set up counter, A level count
  FRQA = 1;
  WAITPNE( pinYMask, pinYMask);                       // Wait for opposite state ready
  PHSA = 0;                                           // Clear count
  WAITPEQ(pinYMask, pinYMask);                        // wait for pulse
  WAITPNE(pinYMask,pinYMask);                         // Wait for pulse to end
  return PHSA; 										  // Return duration as counts

  
  


}




/*

{{

+------------------------------------------------------------------------------------------------------------------------------+
                                                   TERMS OF USE: MIT License                                                                                                              
+------------------------------------------------------------------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation     
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    
modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
is furnished to do so, subject to the following conditions:                                                                   
                                                                                                                              
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
                                                                                                                              
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         
+------------------------------------------------------------------------------------------------------------------------------+
}}      

*/