/**
 * @file mouse.c
 * ICC Propeller mouse driver.
 *
 * Copyright (c) 2008, Steve Denson
 * See end of file for terms of use.
 */
#include <propeller.h>

#include "mouse.h"
#include "MouseArray.h"

static mouse_t gmouse;
static mousecog;

/*
 * start mouse cog
 * See header file for more details.
 */
int mouse_start(int dpin, int cpin)
{
    int id = 0;
    gmouse.dpin = dpin;
    gmouse.cpin = cpin;
    id = cognew_native((void*)MouseArray, (void*)&gmouse);
    wait(10000000); // wait for driver to start
    return id;
}

/*
 * stop mouse cog
 * See header file for more details.
 */
void    mouse_stop(int id)
{
    asm("cogstop %id");
}

/*
 * check if mouse is present
 * See header file for more details.
 */
mouse_type_en mouse_present(void)
{
    return (mouse_type_en) gmouse.present;
}

/*
 * Get mouse buttons state that can be interpreted with mouse_button_en.
 * See header file for more details.
 */
int mouse_buttons(void)
{
    return gmouse.buttons;
}

/*
 * Get mouse x position.
 * See header file for more details.
 */
int mouse_x(void)
{
    return gmouse.x;
}

/*
 * Get mouse y position.
 * See header file for more details.
 */
int mouse_y(void)
{
    return gmouse.y;
}

/*
+------------------------------------------------------------------------------------------------------------------------------+
                                                   TERMS OF USE: MIT License                                                                                                              
+------------------------------------------------------------------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation     
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    
modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
is furnished to do so, subject to the following conditions:                                                                   
                                                                                                                              
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
                                                                                                                              
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         
+------------------------------------------------------------------------------------------------------------------------------+
*/