/*---------------------------------------------------------------------- 
 *      Name:    SPI_MMC.H 
 *      Purpose: SPI mode SD/MMC card interface driver 
 *      Version: V1.03 
 *      Copyright (c) 2006 NXP Semiconductor. All rights reserved. 
 *---------------------------------------------------------------------*/ 
#ifndef __SPI_MMC_H__ 
#define __SPI_MMC_H__
		
/* SPI select pin */ 
#define SPI_SEL      0x00100000 //p0.20-MMC1 select
#define SPI_SEL1	 0x00800000 //p0.23-MMC2 select 
#define	MOSI		 0x00080000//p0.19
#define	MISO		 0x00040000//p0.18
#define SCK 		 0x00020000//p0.17
/*the max number of block 512byte( sector)*/
#define MAX_TRACK	   (512<<20)>>9 //512MB/512=number of block
#define MAX_BLOCK_NUM  (512<<20)>>9 //512MB/512=number of block
 
 
/* The SPI data is 8 bit long, the MMC use 48 bits, 6 bytes */ 
#define MMC_CMD_SIZE  6 
 
/* The max MMC flash size is 256MB */ 
#define MMC_DATA_SIZE 512    /* 16-bit in size, 512 bytes */ 
  
#define MAX_TIMEOUT  0xFF 
 
#define IDLE_STATE_TIMEOUT	  	      65
#define OP_COND_TIMEOUT  	  	      66 
#define SET_BLOCKLEN_TIMEOUT  	      67 
#define WRITE_BLOCK_TIMEOUT   	      68 
#define WRITE_BLOCK_FAIL      		  69 
#define READ_BLOCK_TIMEOUT     	      70 
#define READ_BLOCK_DATA_TOKEN_MISSING 71 
#define DATA_TOKEN_TIMEOUT            72 
#define SELECT_CARD_TIMEOUT       	  73 
#define SET_RELATIVE_ADDR_TIMEOUT     74

void SPI_Init( void );//share between 2 MMC card 
void SPI_Send( unsigned char *Buf, unsigned long Length );//share between 2 MMC card  
void SPI_Receive( unsigned char *Buf, unsigned long Length );//share between 2 MMC card  
unsigned char SPI_ReceiveByte( void );//share between 2 MMC card  
 
int mmc_init(void); 
int mmc_response(unsigned char response);//share between 2 MMC card  
int mmc_read_block(unsigned long block_number); 
int mmc_write_block(unsigned long block_number); 
int mmc_wait_for_write_finish(void);//share between 2 MMC card 
unsigned char mmc_clear(void);
int mmc1_init(void); 
//int mmc1_response(unsigned char response); 
int mmc1_read_block(unsigned long block_number); 
int mmc1_write_block(unsigned long block_number); 
//int mmc1_wait_for_write_finish(void); 
 
#endif  /* __SPI_MMC_H__ */ 


