//==================doing kalmman filter====
#include <math.h>
#include "kalman.h"
#include "ADIS16350.h"

/*---------------for KALMAN Operating----------------------------------*/
/* Our covariance matrix.  This is updated at every time step to
 * determine how well the sensors are tracking the actual state.
 */
float PY[4] ={1,0,0,1};
float PX[4] ={1,0,0,1};

/*two states, the angle and the gyro bias.  As a byproduct of computing
 * the angle, we also have an unbiased angular rate available. */

float 	angleWx_init,angleWy_init;

/*
 * R represents the measurement covariance noise.  In this case,
 * it is a 1x1 matrix that says that we expect 0.3 rad jitter
 * from the accelerometer. standard 0.3
 */
static  float	R_angle_X= 0.03;
static  float	R_angle_Y= 0.03;

/*
 * Q is a 2x2 matrix that represents the process covariance noise.
 * In this case, it indicates how much we trust the acceleromter
 * relative to the gyros.
 */
  float	Q_angle_X	= 0.001;//0.03^2
  float	Q_gyro_X	= 0.003;//0.055^2
  
  float	Q_angle_Y	= 0.001;
  float	Q_gyro_Y	= 0.05;

unsigned char flag_kalman_update;//extern variable to implement kalman function

void Kalman_Filter(signed short AccX,/*input*/
				   signed short AccY,
				   signed short AccZ,
				   signed short GyrX,
				   signed short GyrY,
				   signed short GyrZ,
				   float *angleWx,/*output*/
				   float *rate_Wx,
				   float *q_biasWx,
				   float *angleWy,
				   float *rate_Wy,
				   float *q_biasWy)
{
float pitch_x,yaw_y;
float gyro_x,gyro_y,gyro_z;

	  
	//  if(flag_kalman_update==1)
	  {	
		 flag_kalman_update=0;

	 	 pitch_x=atan2(AccY,AccZ);
		 gyro_x= GyrX*GyroScalar;

         yaw_y = atan2(AccX,AccZ);
		 gyro_y= GyrY*GyroScalar;

		 gyro_z=GyrZ*GyroScalar;

		 kalman_update(gyro_x,pitch_x,Q_angle_X,Q_gyro_X,R_angle_X,angleWx,rate_Wx,q_biasWx,PX);
 		 kalman_update(gyro_y,  yaw_y,Q_angle_Y,Q_gyro_Y,R_angle_Y,angleWy,rate_Wy,q_biasWy,PY);
	  }
}

//---------for kalman filter Area------------
/*kalman filter area */
void kalman_update( float	q_m,/*gyro measurement */
					float angle_m,//acc angle
					float Q_angle,//process noise
				    float Q_gyro,//process noise
				   	float R_angle,/* measurement noise*/
					float *angle, /*output angle*/
					float *rate,//angle velocity update
					float *q_bias,/*output bias*/
					float *P) /*covariance matrix*/
{	
	 float q ;
     float Pdot[4];	 //dP/dt
					
	 float		C_0 = 1;
	/* const float		C_1 = 0; */
	// float		angle_m;
	 float		angle_err;
	 float		PCt_0,PCt_1;// P[2x2]*C'[2x1] =[PCt_0,PCt1]
	 float		E;//[1x1] matrix 	E<1,1> = C<1,2> P<2,2> C'<2,1> + R<1,1>
	 float		K_0,K_1;//Kalman gain [2x1] matrix K=[K_0,K_1]' 
	 float		t_0,t_1;//C<1,2>*P<2,2> as 1x2 matrix  

//==========================state update==================================
	 /* Unbias our gyro */
    q= q_m - *q_bias;
   	/*	Pdot = A*P + P*A' + Q
	 Q=[ Q_angle  0,
	       0 Q_gyro] */
	
	Pdot[0]=Q_angle - P[1]-P[2];//P[0][1] - P[1][0];/* 0,0 */
	Pdot[1]= -P[3];//P[1][1];	  /* 0,1 */
	Pdot[2]= -P[3];//P[1][1];	/* 1,0 */
	Pdot[3]=  Q_gyro;	/* 1,1 */

   	/* Store our unbias gyro estimate */
	*rate = q;
	*angle += q * dt;
	/* Update the covariance matrix */
	/*P[0][0]*/P[0] += Pdot[0] * dt;
	/*P[0][1]*/P[1] += Pdot[1] * dt;
	/*P[1][0]*/P[2] += Pdot[2] * dt;
	/*P[1][1]*/P[3] += Pdot[3] * dt;


//========end state update and start kalman update======= 

/* Compute our measured angle and the error in our estimate */
//observer equation   angle_m = C*X + N(R,0) which X=[angle bias_gyro]'
	 angle_err = angle_m - *angle;//-angle_init;
//compute matrix P*C'=[PCt_0,PCt_1] as  [2x1] matrix
	PCt_0 = C_0 * P[0];//P[0][0]; /* + C_1 * P[0][1] = 0 */
	PCt_1 = C_0 * P[2];//P[1][0]; /* + C_1 * P[1][1] = 0 */
//	E<1,1> = C<1,2> P<2,2> C'<2,1> + R<1,1>
 	E =	R_angle	+ C_0 * PCt_0;/*	+ C_1 * PCt_1 = 0 */
// Compute the Kalman filter gains
// 	K = P C' inv(E)  which PC' calculate above
//	K<2,1> = P<2,2> C'<2,1> inv(E)<1,1>
//because  E is <1,1>, so the inverse of E is just 1/E.
	K_0 = PCt_0 / E;
	K_1 = PCt_1 / E;
//Update covariance matrix.  Again, from the Kalman filter paper:
//	P = P - K C P
//	P<2,2> -= K<2,1> C<1,2> P<2,2>
// first compute t<1,2> = C P.  Note that:
//	t[0,0] = C[0,0] * P[0,0] + C[0,1] * P[1,0]=PCt_0  (C_1=0)
	t_0 = PCt_0; /* C_0 * P[0][0] + C_1 * P[1][0] */
	t_1 = C_0 *P[1]; //P[0][1]; /* + C_1 * P[1][1]  = 0 */

	/*P[0][0]*/P[0] -= K_0 * t_0;
	/*P[0][1]*/P[1] -= K_0 * t_1;
	/*P[1][0]*/P[2] -= K_1 * t_0;
	/*P[1][1]*/P[3] -= K_1 * t_1;
//Update our state estimate. 
//X += K * err;
//X<2x1> = X<2x1> + K<2,1> * err<1,1>
// err is a measurement of the difference in the measured state
// and the estimate state which is the difference between 
//the two accelerometer measured angle and our estimated  angle.

	*angle	+= K_0 * angle_err;
	*q_bias	+= K_1 * angle_err;
}
