#include "math.h"
#include "kalman_attitude_3truc.h"
#include "ADIS16350.h"
#include "timer.h"
#include "serial0.h"
#include "stdio.h"
#include <float.h>
 //--------------------data for IMU ADIS16350------------------------------------------
unsigned short ADIS16350_rbuff[7];
signed short ADIS16350_data[7];	 //ax,ay,az,wx,wy,wz,vcc

//state variables
float corrected_rates[3]; //p,q,r in rad/sec
//kalman estimates
float angle_reference_from_accels[3] = {0.0,0.0,0.0}; //accelerometer based angle estimate
float angle_estimate[3]; //pretty obvious (current state est)
float gyro_bias[3]={0.0,0.0,0.0};
float last_time_tick;
float dt ;//period between two times kalman update
//covariance update matrix
float P[3][2][2] = {
	{
		{ 1, 0 },
		{ 0, 1 },
	},
	{
		{ 1, 0 },
		{ 0, 1 },
	},
	{
		{ 1, 0 },
		{ 0, 1 },
	}
};

//accelerometer measurement covariance noise
const float R_angle = 0.3;
const float Q_angle = 0.001;
const float Q_gyro = 0.003;

//accelerometer bias estimate (updated in kalman_update)
float gyro_bias[3];


//-----------------here is the function-------------------
/******************************************************************************
	ESTIMATE ATTITUDE WITH A KF. NOT REALLY IMPLEMENTED YET, BUT THE
	NAME SUGGESTS WHAT THE INTENTION IS.
*******************************************************************************/
void gyro2angle(void){
	int i;
	float gyro_components[3]; // dot values from matrix
	float sin_roll,tan_pitch, cos_roll, cos_pitch;
	
	//values were in euler. trig could never work right. try matrix again now
	//that we got our internals 100% radians
	sin_roll  = sin(angle_estimate[0]);
	cos_roll  = cos(angle_estimate[0]);
	cos_pitch = cos(angle_estimate[1]);
	tan_pitch = tan(angle_estimate[1]);
	
	/***************************************************************************
		ESTIMATOR MATRIX
		---------------------------------------
		[phi_dot	]	[1	   sin(phi)tan(theta)	cos(phi)*tan(theta)	]	[P]
		[theta_dot	] = [0		cos(phi)				  -sin(phi)	]	] *	[Q]
		[psi_dot	]	[0	sin(phi)/cos(theta)		cos(phi)/cos(theta)	]	[R]
	***************************************************************************/
	
	//angle-estimate from matrix
	gyro_components[0] = corrected_rates[0]*1 + corrected_rates[1]*sin_roll*tan_pitch + corrected_rates[2]*cos_roll*tan_pitch;
	gyro_components[1] = corrected_rates[0]*0 + corrected_rates[1]*cos_roll + corrected_rates[2]*-sin_roll;
	gyro_components[2] = corrected_rates[0]*0 + corrected_rates[1]*(sin_roll/cos_pitch) + corrected_rates[2]*(cos_roll*cos_pitch);

	for(i=0;i<3;i++){
		//state estimate using integration (rate*time = angle)
		// the filter seems to take a while to converge to correct values after fast jitters
		// i think i may have forgotten to remove the estimate error value here
		angle_estimate[i] += ((dt * gyro_components[i]) - gyro_bias[i]);
	}
//for debug only	
/*
	printf("Angle estimate (INTERNAL): %f %f %f\n", angle_estimate[0], angle_estimate[1], angle_estimate[2]);
		
	printf("Corrected Roll: %4.8f Pitch: %4.8f Yaw: %4.8f\n",
		corrected_rates[0],
		corrected_rates[1],
		corrected_rates[2]);
	printf("Time ref: %4.8lf Sample freq: %4.8lf\n", dt, 1/dt);
	printf("Accelerometer state est: Roll: %4.8f Pitch: %4.8f Yaw: %4.8f\n",
		angle_reference_from_accels[0],
		angle_reference_from_accels[1],
		angle_reference_from_accels[2]);
	printf("Kalman filtered output: Roll: %4.8f Pitch: %4.8f Yaw: %4.8f\n",
		angle_estimate[0],
		angle_estimate[1],
		angle_estimate[2]);
	printf("Kalman filtered output degs: Roll: %4.8f Pitch: %4.8f Yaw: %4.8f\n",
		angle_estimate[0]*r2d,
		angle_estimate[1]*r2d,
		angle_estimate[2]*r2d);	 */
}

void state_update(void) //called on new gyro output available
{
	int i;
	float Pdot[4];
	//state update really is just a progression of the internal covariance estimate.
	//it does not use any imu output, only the time ref.
	//the integration, unlike what happens with rotomotion's code, is done in gyro2angle.
	//this allows us to track cross-axis interference :)
	
	for(i=0;i<3;i++){
		
		Pdot[0]=Q_angle - P[i][0][1] - P[i][1][0];	/* 0,0 */
		Pdot[1]=- P[i][1][1];			        	/* 0,1 */
		Pdot[2]=- P[i][1][1];			        	/* 1,0 */
		Pdot[3]=	Q_gyro	;			    		/* 1,1 */
	

		/* Update the covariance matrix */
		P[i][0][0] += Pdot[0] * dt;
		P[i][0][1] += Pdot[1] * dt;
		P[i][1][0] += Pdot[2] * dt;
		P[i][1][1] += Pdot[3] * dt;
	}
}

void kalman_update(int axis_index) // i is axis index
{
	float angle_err;
	float C_0 = 1.0;
	float PCt_0;
	float PCt_1;
	float E;
	float	K_0;
	float	K_1;
	float	t_0;
	float	t_1;
	angle_err = angle_reference_from_accels[axis_index] - angle_estimate[axis_index];
	
	 PCt_0 = C_0 * P[axis_index][0][0];
	 PCt_1 = C_0 * P[axis_index][1][0];
			
	//error estimate
	 E = (float)R_angle + (float)C_0 * PCt_0;
	 //if(abs(E)-1000>FLT_MAX)
	 //E=FLT_MAX-100;
	 //printf("E%d=%04.3f\n",axis_index,E);
	 //getkey();

	 K_0 = PCt_0 / E;
	 K_1 = PCt_1 / E;
			
	 t_0 = PCt_0;
	 t_1 = C_0 * P[axis_index][0][1];

	P[axis_index][0][0] -= K_0 * t_0;
	P[axis_index][0][1] -= K_0 * t_1;
	P[axis_index][1][0] -= K_1 * t_0;
	P[axis_index][1][1] -= K_1 * t_1;
		
	angle_estimate[axis_index] += K_0 * angle_err;
	gyro_bias[axis_index] += K_1 * angle_err;
}

/*******************************************************************
	TRANSFORM ACCELEROMETER DATA INTO ANGLES (RADIANS)
********************************************************************/
void accel2angle(void){
	float g;
	double g2;
	// Create reference attitute estimation for matrix.
	
	// when perpendicular to level, output accuracy decreases 17.5mg per angle of tilt, so
	// use last angle to calculate scale factor change
	// real_accel[i] = accels[i] - 0.0175*angle_estimate[i]
	
/*	accels[0] = ((raw_accels[0] - raw_accels_null[0])*a2v)/0.312;
	accels[1] = ((raw_accels[1] - raw_accels_null[1])*a2v)/0.312;
	accels[2] = ((raw_accels[2] - raw_accels_null[2])*a2v)/0.312; //was 1  */



	// Calculate g ( ||g_vec|| )
	g2 = (double)(ADIS16350_data[0]*ADIS16350_data[0] + ADIS16350_data[1]*ADIS16350_data[1] + ADIS16350_data[2]*ADIS16350_data[2]);
	g  =(float)sqrt(g2);
	
	//values in radians
	angle_reference_from_accels[0] = (float)-asin(ADIS16350_data[0] / g);
	angle_reference_from_accels[1] = (float)asin(ADIS16350_data[1] / g);
	angle_reference_from_accels[2] = 0.0;//compass_heading*d2r; //gps_heading or compass;
	//printf("%04.3f,%04.3f,%04.3f\n\r",angle_reference_from_accels[0],angle_reference_from_accels[1],angle_reference_from_accels[2]);
	//getkey();
}

char read_IMU(void)
{
	char status;
	float  current_time;
	SPI_read16_ADIS16350(ADIS16350_rbuff);
	status = map_14bit_to_16bit_format(ADIS16350_rbuff,ADIS16350_data,7);
	current_time=(float) get_system_tick0();
	dt=	(float) (current_time-last_time_tick)/1000000.00;	 //each tick is equivalent to 1us= 1 ppm  seccond

	if(dt<=0)
	{
 		dt=	(float)( -last_time_tick+current_time+1)/1000000.00+MAX_TIMER_TICK;
	}
	last_time_tick=current_time;
//	printf("dt= %1.7f\n",dt);	
//	getkey();
//	set_system_tick0(0);
//	last_time_tick=0;
    return status;
}

void Gyro_rate_estimate(void)
{
corrected_rates[0]=ADIS16350_data[3]*Gyro_scale_rad- gyro_bias[0];
corrected_rates[1]=ADIS16350_data[4]*Gyro_scale_rad- gyro_bias[1];
corrected_rates[2]=ADIS16350_data[5]*Gyro_scale_rad- gyro_bias[2];
}
//-------------------------kalman--------------------------------
char do_kalman_filter(void)
{

	if(read_IMU()==0)  //successfully read 
	{
	  Gyro_rate_estimate();
	  gyro2angle();
	  accel2angle();
	  
	  state_update();
	  kalman_update(1);
	  kalman_update(2);
	//  kalman_update(3);
	  
	}
	else
	{
	  printf("IMU invalid data\n");
	  return 1;//error
	}
	return 0;
}
