// Synth.c use cog counters to generate output frequency.
// Use Catalina C compiler.
//   Flashes leds 16 and 17 on DEMO  board at 10 and 2 Hz respectively
//   using A and B counters of current Cog.
//
//  Author: Juan Carlos Orozco
//  Copyright (c) 2010 Juan Carlos Orozco ACELAB LLC
//  http://www.acelab.com
//  Industrial Automation

//  License: See end of file for terms of use.

// Ported from Synth.spin:
//  Synth.spin Original Author: Chip Gracey, Modified by Beau Schwabe

#include <catalina_cog.h>

unsigned _ctra(unsigned val);
unsigned _ctrb(unsigned val);
unsigned _frqa(unsigned val);
unsigned _frqb(unsigned val);

unsigned fraction(unsigned a, unsigned b, int shift){
    unsigned f = 0;
    int i;

    if(shift > 0) a <<= shift;
    if(shift < 0) b <<= -shift;

    for(i=0; i<32; i++){
        f <<= 1;
        if(a >= b){
            a -= b;
            f++;
        }
        a <<= 1;
    }
    return f;
}

// This function is based and adapted from (public domain):
// http://graphics.stanford.edu/~seander/bithacks.html#IntegerLogObvious
int highest_bit(unsigned v){
    register unsigned int r; // result of log2(v) will go here
    register unsigned int shift;

    r =     (v > 0xFFFF) << 4; v >>= r;
    shift = (v > 0xFF  ) << 3; v >>= shift; r |= shift;
    shift = (v > 0xF   ) << 2; v >>= shift; r |= shift;
    shift = (v > 0x3   ) << 1; v >>= shift; r |= shift;
                                            r |= (v >> 1);
    if(v==0){ r=0; }
    else{ r++; }
    return(r);                                       
}   

void synth(int AB, int pin, unsigned Freq){
    unsigned frq;
    unsigned ctr;
    unsigned pin_;
    unsigned s, d;

    if(Freq <= 0) Freq = 1;
    if(Freq >= 128000000) Freq = 127999999;
    if(Freq<500000){
        ctr = (4 << 26);
        s = 1;
    }
    else{
        ctr = (2 << 26);
        d = highest_bit((Freq-1)/1000000);
        s = 4 - d;  // determine shift
        ctr |= (d << 23);
    }
    frq = fraction(Freq, _clockfreq(), s);
    ctr |= pin;

    pin_ = 1<<pin;
    
    if(AB){
        _ctra(ctr);
        _frqa(frq);
        _dira(pin_, pin_);
    }
    else{
        _ctrb(ctr);
        _frqb(frq);
        _dira(pin_, pin_);
    }
}

void main() {
    // Flash leds 16 and 17 on DEMO  board at 10 and 2 Hz respectively
    // using A and B counters of current Cog.
    synth(1, 16, 10);
    synth(0, 17, 2);

    while(1){
    }
}


//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:                                                                   
//                                                                                                                              
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//                                                                                                                              
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         
//

