//============================================================================
//  TSL230_Test.c                 Allen Marincak            Ver 1.0   Mar 2009
//
//      Test program for the TSL230R Driver.
//
//  Copyright (c) 2009, Allen Marincak       See end of file for terms of use.
//============================================================================

#include <propeller.h>
#include "fdserial.h"
#include "tsl230.h"

//----------------------------------------------------------------------------
//  main()
//
//  Demo the TSL230.
//----------------------------------------------------------------------------
void main( void )
{
    unsigned long sample, sens, count, time, idx1, idx2;

    FdSerial_start(31,30,0,9600);           // start serial driver (via USB)
    TSL230_Start( 25, 26, 1, 1 );           // start TSL230 driver
    
    time = 80000000;
    count = CNT;
    count += time;
    
    for(;;)
    {
    
        TSL230_SetSensitivity( 0 );             // restore autosensitivity
        FdSerial_str( "-----------------------------\r\n" );
        for ( idx1 = 0; idx1 < 10; idx1++ )
        {
            WAITCNT( count, time );             // wait for a second
    
            TSL230_GetSampleSens( &sample, &sens );
            FdSerial_str( "Sens (A): " );
            FdSerial_dec( sens );
            FdSerial_str( "   Sample: " );
            FdSerial_dec( sample );
            FdSerial_str( "\r\n" );
        }
        
        for ( idx2 = 3; idx2 > 0 ; idx2-- )
        {
            TSL230_SetSensitivity( idx2 );      // set manual sensitivity
            FdSerial_str( "-----------------------------\r\n" );
            for ( idx1 = 0; idx1 < 10; idx1++ )
            {
                WAITCNT( count, time );         // wait for a second
    
                TSL230_GetSampleSens( &sample, &sens );
                FdSerial_str( "Sens (M): " );
                FdSerial_dec( sens );
                FdSerial_str( "   Sample: " );
                FdSerial_dec( sample );
                FdSerial_str( "\r\n" );
            }
        }        
    }
}     

//============================================================================
//                        TERMS OF USE: MIT License                                                       
//============================================================================
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
//                                       
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
//============================================================================
