/**
 * @file timer_test.c
 *
 * This file tests the timer module.
 *
 * Copyright (c) 2008, Steve Denson
 * See end of file for terms of use.
 */
#include "sllist.h"
#include "timer.h"
#include <stdlib.h>
#include <stdio.h>

#ifdef NOPROP
// NOPROP case is for building with GNU or MSVC++
void asm(char* s) {}
#define  dinit()
#else
// PROP case
#include <propeller.h>
#include <propclock.h>
#include <vga_text.h>
#define  dinit()     vgaText_start(16);
#endif

int  gmheap[0x1000];

void scope_trigger(void)
{
    DIRA |= 2;
    OUTA ^= 2;
}

void my_timer_proc(void)
{
    DIRA |= 1;
    OUTA ^= 1;
}

/**
 * main ...
 * - create a 5ms timer proc list
 * - add 800 timer procs
 * - let timer run a while
 * - destroy timer
 */
void main(void)
{
    int  ii = 0;  	 
    int  len = 100;
    
    _NewHeap(gmheap, gmheap+sizeof(gmheap)-1);

    dinit();
  
    printf("Hello World.\n");

    while(len--)
    {
        gtimer = timer_create(5);
        printf("Create timer. COGID %d\n", gtimer->id);
    
        if(!gtimer)  {
            printf("timer_create failed.\n");
            while(1);
        }
    
        printf("Add timer procs.\n");

        timer_disable(gtimer);
        timer_add(gtimer,&scope_trigger);
        for(ii = 0; ii < 800; ii++) {
            if(!timer_add(gtimer,&my_timer_proc)) {
                // fails around 835 for lack of memory
                printf("timer_add %d failed.\n",ii);
                while(1);
            }
        }
        timer_enable(gtimer);

        printf("Timer Started.\n");
        sleep(4);
		        
        timer_destroy(gtimer);
        printf("Timer Destroyed.\n");
        sleep(1);
    }

    printf("Goodbye World.\n");

    while(1);
}

/*
+------------------------------------------------------------------------------------------------------------------------------+
                                                   TERMS OF USE: MIT License                                                                                                              
+------------------------------------------------------------------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation     
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    
modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
is furnished to do so, subject to the following conditions:                                                                   
                                                                                                                              
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
                                                                                                                              
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         
+------------------------------------------------------------------------------------------------------------------------------+
*/