/* =========================================================================
 * LTC1860 12-bit analog digital converter driver 
 * -------------------------------------------------------------------------
 * File:    ltc1860.h
 * Author:  Pavel Pinkas, Ph. D.
 * Version: 001 (released 2009-08-06)
 *
 * Copyright (c) 2009 Pavel Pinkas, Ph. D. <pavelp@dnastar.com>
 * See end of file for terms of use. 
 * -------------------------------------------------------------------------
 * CHANGELOG:
 * 2009/08/06: V001 - Initial Release  
 * ========================================================================= */
 
/* -------------------------------------------------------------------------
 * Internal implementation details   
 * ------------------------------------------------------------------------- */
#include "ltc1860_private.h"

/* -------------------------------------------------------------------------
 * Device configuration structure  
 * ------------------------------------------------------------------------ */
typedef struct 
{
    // USER 
	unsigned short CONV;	// pin to start conversion
	unsigned short SDO;     // data-output pin
	unsigned short SCK;     // clock pin
    
    // private data (leave alone)
    LTC1860_private inner;
} LTC1860;

/* -------------------------------------------------------------------------
 * Public functions (to be used by user)  
 * ------------------------------------------------------------------------- */
void ltc1860_setup (						// setup device
	LTC1860 *dev,            
    unsigned short convPin,					// CONVersion pin
    unsigned short sckPin,					// SCK (clock) pin
    unsigned short sdoPin);					// SDO (data) pin
    
void ltc1860_reset (LTC1860 *dev);			// re-initialize the device 
                   
unsigned short ltc1860_read (LTC1860 *dev); // make one raw reading

void ltc1860_set_range (                    // set range of values (voltages)
	LTC1860 *dev,
    float min,								// the lowest possible value
    float max);								// the highest possible value

float ltc1860_convert (LTC1860 *dev);       // read and convert 

void ltc1860_set_converter (
	LTC1860 *dev,
    LTC1860_Converter function);
    
void ltc1860_raw (LTC1860 *dev); 			// clear any set conversion

/* ===========================================================================
 *                         TERMS OF USE: MIT License
 * ---------------------------------------------------------------------------
 * Copyright (c) 2009 Pavel Pinkas, Ph. D. <pavelp@dnastar.com>
 * ---------------------------------------------------------------------------
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 *
 * ======================================================================== */
 
