/* =========================================================================
 * LTC1860 12-bit analog digital converter driver 
 * -------------------------------------------------------------------------
 * File:    ltc1860_private.h
 * Author:  Pavel Pinkas, Ph. D.
 * Version: 001 (released 2009-08-06)
 *
 * Copyright (c) 2009 Pavel Pinkas, Ph. D. <pavelp@dnastar.com>
 * See end of file for terms of use. 
 * -------------------------------------------------------------------------
 * CHANGELOG:
 * 2009/07/31: V001 - Initial Release  
 * ========================================================================= */

/* -------------------------------------------------------------------------
 * Converter function prototype
 * ------------------------------------------------------------------------- */
typedef float (*LTC1860_Converter) (unsigned short);
 
/* -------------------------------------------------------------------------
 * Constants  
 * ------------------------------------------------------------------------- */
#define CONVERTER_RESOLUTION 12
#define CONVERTER_RANGE 4096

#define TCONV_DURATION 4

/* -------------------------------------------------------------------------
 * Conversion type   
 * ------------------------------------------------------------------------- */
typedef enum   
{
	NO_CONVERSION,
    RANGE_CONVERSION,
    CUSTOM_CONVERTER
} ConversionType;

/* -------------------------------------------------------------------------
 * Internal part of the LTC1860 structure 
 * ------------------------------------------------------------------------- */
typedef struct 
{    
    long mask;				// mask for reading output 
    
    long tconv_cycles;		// how many cycles for TCONV

    float min;				// bottom of the converter range
    float max;              // top of the converter range 
    
    LTC1860_Converter cf;   // converter function 
    ConversionType ctype;   // conversion type  
} LTC1860_private;

/* ===========================================================================
 *                         TERMS OF USE: MIT License
 * ---------------------------------------------------------------------------
 * Copyright (c) 2009 Pavel Pinkas, Ph. D. <pavelp@dnastar.com>
 * ---------------------------------------------------------------------------
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 *
 * ======================================================================== */

