/* =========================================================================
 * MCP3201 :: 12-bit, 1-channel, analog digital converter driver 
 * -------------------------------------------------------------------------
 * File:    mcp3201.h
 * Author:  Pavel Pinkas, Ph. D.
 * Version: 001 (released 2009-08-28)
 *
 * Copyright (c) 2009 Pavel Pinkas, Ph. D. <pavelp@dnastar.com>
 * See end of file for terms of use. 
 * -------------------------------------------------------------------------
 * CHANGELOG:
 * 2009/08/28: V001 - Initial Release   
 * ========================================================================= */

#ifndef __MCP3201_H__
#define __MCP3201_H__

/* -------------------------------------------------------------------------
 * Internal details (no concern to the user)
 * ------------------------------------------------------------------------- */
#include "mcp3201_internal.h"

/* -------------------------------------------------------------------------
 * Device description structure 
 * ------------------------------------------------------------------------- */
typedef struct _mcp3201_
{
	unsigned char CLKPin;			// clock pin
    unsigned char CSPin;			// chip-select pin
    unsigned char DOUTPin;			// data-out pin 

    MCP3201_internal internal;      // internal structure (hidden from user)
} MCP3201;

/* -------------------------------------------------------------------------
 * Functions callable by the user
 * ------------------------------------------------------------------------- */
void mcp3201_setup (				// setup the device 
	MCP3201 *dev,					// device description
    unsigned char csPin,			// chip-select pin 
    unsigned char clkPin,			// clock pin
    unsigned char doutPin);			// data-out pin
   
void mcp3201_reset (				// reset/restart the device 
	MCP3201 *dev);					// device description
 
unsigned short mcp3201_read (		// read a channel
	MCP3201 *dev);					// device description 
    
float mcp3201_convert (				// read and convert
	MCP3201 *dev);					// device description
    
void mcp3201_set_range (			// set conversion range 
	MCP3201 *dev,					// device description
	float min,						// minimimum measurable value
    float max);						// maximum measurable value
    
void mcp3201_set_converter (		// set custom converter 
	MCP3201 *dev,					// device description
	MCP3201_Converter cf);			// converter function
    
void mcp3201_raw (					// clear any set converter 
	MCP3201 *dev);					// device description			

#endif // __MCP3201_H__
   
/* ===========================================================================
 *                         TERMS OF USE: MIT License
 * ---------------------------------------------------------------------------
 * Copyright (c) 2009 Pavel Pinkas, Ph. D. <pavelp@dnastar.com>
 * ---------------------------------------------------------------------------
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 *
 * ======================================================================== */
 
