/* =========================================================================
 * MCP3201 :: 12-bit, 1-channel, analog digital converter driver 
 * -------------------------------------------------------------------------
 * File:    mcp3201.h
 * Author:  Pavel Pinkas, Ph. D.
 * Version: 001 (released 2009-08-27)
 *
 * Copyright (c) 2009 Pavel Pinkas, Ph. D. <pavelp@dnastar.com>
 * See end of file for terms of use. 
 * -------------------------------------------------------------------------
 * CHANGELOG:
 * 2009/08/27: V001 - Initial Release   
 * ========================================================================= */

 #ifndef __MCP3201_INTERNAL_H__
 #define __MCP3201_INTERNAL_H__

/* -------------------------------------------------------------------------
 * Data exchange details 
 * ------------------------------------------------------------------------- */
#define DATA_SIZE			12		      // 12 bits of data
#define ADC_RANGE			0xFFF	      // maximum value returned by ADC

#define SAMPLE_HOLD_CLOCKS  2             // 2 clocks needed for sample&hold

/* -------------------------------------------------------------------------
 * Converter details 
 * ------------------------------------------------------------------------- */
 typedef float (*MCP3201_Converter) (unsigned short); // converter prototype
 
 typedef enum 										  // conversion type 
 {
 	NO_CONVERSION,								      // no conversion, raw
    RANGE,                                            // min .. max range
    CUSTOM_FUNCTION                                   // custom converter 
 } ConversionType;
 
/* -------------------------------------------------------------------------
 * Internal details of device description structure 
 * ------------------------------------------------------------------------- */
typedef struct _mcp3201_internal_
{
	unsigned char config;			// configuration byte 
    unsigned long mask;				// precomputed mask for reading data

    MCP3201_Converter cf;			// converter function
    ConversionType ctype;           // conversion type 
    
    float min;						// conversion range minimum (if used)
    float max; 						// conversion range maximum (if used)
} MCP3201_internal;

#endif // __MCP3201_INTERNAL_H__

/* ===========================================================================
 *                         TERMS OF USE: MIT License
 * ---------------------------------------------------------------------------
 * Copyright (c) 2009 Pavel Pinkas, Ph. D. <pavelp@dnastar.com>
 * ---------------------------------------------------------------------------
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 *
 * ======================================================================== */
 
