/* =========================================================================
 * MCP3202 :: 12-bit, 2-channel, analog digital converter driver 
 * -------------------------------------------------------------------------
 * File:    mcp3202.h
 * Author:  Pavel Pinkas, Ph. D.
 * Version: 001 (released 2009-08-16)
 *
 * Copyright (c) 2009 Pavel Pinkas, Ph. D. <pavelp@dnastar.com>
 * See end of file for terms of use. 
 * -------------------------------------------------------------------------
 * CHANGELOG:
 * 2009/08/16: V001 - Initial Release   
 * ========================================================================= */

#ifndef __MCP3202_H__
#define __MCP3202_H__

/* -------------------------------------------------------------------------
 * Internal details (no concern to the user)
 * ------------------------------------------------------------------------- */
#include "mcp3202_internal.h"

/* -------------------------------------------------------------------------
 * Configuration constants
 * ------------------------------------------------------------------------- */
typedef enum 
{
	MODE_SINGLE 	  = 0b00000100,
    MODE_DIFFERENTIAL = 0b00000000
} EMode;

typedef enum 
{
	CHANNEL_0		  = 0b00000000,
    CHANNEL_1         = 0b00000010,
    SIGN_POSITIVE     = 0b00000000,
    SIGN_NEGATIVE     = 0b00000010
} EChannel;


typedef enum
{
	MSB_FIRST		  = 0b00000001,
    LSB_FIRST         = 0b00000000
} EBitOrder;

/* -------------------------------------------------------------------------
 * Device description structure 
 * ------------------------------------------------------------------------- */
typedef struct _mcp3202_
{
	unsigned char CLKPin;			// clock pin
    unsigned char CSPin;			// chip-select pin
    unsigned char DOUTPin;			// data-out pin 
    unsigned char DINPin;			// data-in pin 
    
	EMode mode;				        // mode: single or differential
    EBitOrder msbf;				    // MSB / LSB first

    MCP3202_internal internal;      // internal structure (hidden from user)
} MCP3202;

/* -------------------------------------------------------------------------
 * Functions callable by the user
 * ------------------------------------------------------------------------- */
void mcp3202_setup (				// setup the device 
	MCP3202 *dev,					// device description
    unsigned char csPin,			// chip-select pin 
    unsigned char clkPin,			// clock pin
    unsigned char dinPin,			// data-in pin
    unsigned char doutPin,			// data-out pin
    EMode mode, 			        // mode (single or differential)
    EBitOrder msbf);  			    // MSB or LSB first 
   
void mcp3202_reset (				// reset/restart the device 
	MCP3202 *dev);					// device description
 
unsigned short mcp3202_read (		// read a channel
	MCP3202 *dev,					// device description 
    EChannel channel);	            // channel to read
    
float mcp3202_convert (				// read and convert
	MCP3202 *dev,					// device description
    EChannel channel);				// channel to read
    
void mcp3202_set_range (			// set conversion range 
	MCP3202 *dev,					// device description
	float min,						// minimimum measurable value
    float max);						// maximum measurable value
    
void mcp3202_set_converter (		// set custom converter 
	MCP3202 *dev,					// device description
	MCP3202_Converter cf);			// converter function
    
void mcp3202_raw (					// clear any set converter 
	MCP3202 *dev);					// device description			

#endif // __MCP3202_H__
   
/* ===========================================================================
 *                         TERMS OF USE: MIT License
 * ---------------------------------------------------------------------------
 * Copyright (c) 2009 Pavel Pinkas, Ph. D. <pavelp@dnastar.com>
 * ---------------------------------------------------------------------------
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 *
 * ======================================================================== */
 
